@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016 Olly Betts
+ * Copyright 2002-2025 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -59,6 +59,13 @@
 using namespace Glass;
 using namespace std;
 
+[[noreturn]]
+static void
+throw_corrupt(const char* message)
+{
+    throw Xapian::DatabaseCorruptError(message);
+}
+
 //#define BTREE_DEBUG_FULL 1
 #undef BTREE_DEBUG_FULL
 
@@ -574,6 +581,7 @@ GlassTable::compact(uint8_t * p)
 	    LeafItem item(p, c);
 	    int l = item.size();
 	    e -= l;
+	    if (e < 0) throw_corrupt("Leaf item size extends outside block");
 	    memcpy(b + e, item.get_address(), l);
 	    LeafItem_wr::setD(p, c, e);  /* reform in b */
 	}
@@ -583,12 +591,14 @@ GlassTable::compact(uint8_t * p)
 	    BItem item(p, c);
 	    int l = item.size();
 	    e -= l;
+	    if (e < 0) throw_corrupt("Branch item size extends outside block");
 	    memcpy(b + e, item.get_address(), l);
 	    BItem_wr::setD(p, c, e);  /* reform in b */
 	}
     }
     memcpy(p + e, b + e, block_size - e);  /* copy back */
     e -= dir_end;
+    if (e < 0) throw_corrupt("Items overlap with item pointers");
     SET_TOTAL_FREE(p, e);
     SET_MAX_FREE(p, e);
 }
@@ -1346,8 +1356,13 @@ GlassTable::add(const string& key, const string& tag, bool already_compressed)
     /* FIXME: sort out this error higher up and turn this into
      * an assert.
      */
-    if (m >= BYTE_PAIR_RANGE)
-	throw Xapian::UnimplementedError("Can't handle insanely large tags");
+    if (m >= BYTE_PAIR_RANGE) {
+	string message = "Btree tag entry of size ";
+	message += str(tag_size);
+	message += " is too large to store - "
+	     "increase the block size to raise this limit";
+	throw Xapian::UnimplementedError(message);
+    }
 
     size_t o = 0;                     // Offset into the tag
     size_t residue = tag_size;        // Bytes of the tag remaining to add in
@@ -1512,19 +1527,15 @@ GlassTable::read_tag(Glass::Cursor * C_, string *tag, bool keep_compressed) cons
 
     tag->resize(0);
 
-    bool first = true;
-    bool compressed = false;
+    LeafItem item(C_[0].get_p(), C_[0].c);
+    bool compressed = item.get_compressed();
     bool decompress = false;
+    if (compressed && !keep_compressed) {
+	comp_stream.decompress_start();
+	decompress = true;
+    }
+
     while (true) {
-	LeafItem item(C_[0].get_p(), C_[0].c);
-	if (first) {
-	    first = false;
-	    compressed = item.get_compressed();
-	    if (compressed && !keep_compressed) {
-		comp_stream.decompress_start();
-		decompress = true;
-	    }
-	}
 	bool last = item.last_component();
 	if (decompress) {
 	    // Decompress each chunk as we read it so we don't need both the
@@ -1542,6 +1553,7 @@ GlassTable::read_tag(Glass::Cursor * C_, string *tag, bool keep_compressed) cons
 	if (!next(C_, 0)) {
 	    throw Xapian::DatabaseCorruptError("Unexpected end of table when reading continuation of tag");
 	}
+	item.init(C_[0].get_p(), C_[0].c);
     }
     // At this point the cursor is on the last item - calling next will move
     // it to the next key (GlassCursor::read_tag() relies on this).
