@@ -1,7 +1,7 @@
 /** @file
  * @brief Command line search tool using Xapian::QueryParser.
  */
-/* Copyright (C) 2004-2022 Olly Betts
+/* Copyright (C) 2004-2026 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -84,6 +84,7 @@ static const tab_entry flag_tab[] = {
     { "cjk_ngram", Xapian::QueryParser::FLAG_CJK_NGRAM },
     { "default", Xapian::QueryParser::FLAG_DEFAULT },
     { "lovehate", Xapian::QueryParser::FLAG_LOVEHATE },
+    { "ngrams", Xapian::QueryParser::FLAG_NGRAMS },
     { "no_positions", Xapian::QueryParser::FLAG_NO_POSITIONS },
     { "partial", Xapian::QueryParser::FLAG_PARTIAL },
     { "phrase", Xapian::QueryParser::FLAG_PHRASE },
@@ -139,6 +140,14 @@ static const tab_entry wt_tab[] = {
     { "trad",	WEIGHT_TRAD }
 };
 
+static const tab_entry stem_strategy_tab[] = {
+    { "all", Xapian::QueryParser::STEM_ALL },
+    { "all_z", Xapian::QueryParser::STEM_ALL_Z },
+    { "none", Xapian::QueryParser::STEM_NONE },
+    { "some", Xapian::QueryParser::STEM_SOME },
+    { "some_full_pos", Xapian::QueryParser::STEM_SOME_FULL_POS }
+};
+
 /** The number of spaces to indent by in print_table.
  *
  *  This needs to match the indent in the help message in show_usage() below.
@@ -146,7 +155,7 @@ static const tab_entry wt_tab[] = {
 #define INDENT \
 "                                    "
 
-/** Print string from a string to integer mapping table.
+/** Print strings from a string to integer mapping table.
  *
  *  @param table  Array of tab_entry in ascending string order.
  */
@@ -170,6 +179,44 @@ print_table(const T& table)
     return '\n';
 }
 
+/** Print available stemmers, line wrapped. */
+static char
+print_stemmers()
+{
+    size_t pos = 256;
+    const string langs = Xapian::Stem::get_available_languages();
+    size_t p = 0;
+    while (p != string::npos) {
+	size_t space = langs.find(' ', p);
+	size_t len = (space != string::npos) ? space - p : langs.size() - p;
+	if (pos < 256) cout << ',';
+	if (pos + len >= 78) {
+	    cout << "\n" INDENT;
+	    pos = sizeof(INDENT) - 2;
+	} else {
+	    cout << ' ';
+	}
+	cout << string(langs.data() + p, len);
+	pos += len + 2;
+	p = space;
+	if (p != string::npos) ++p;
+    }
+    return '\n';
+}
+
+/** List strings from a string to integer mapping table, one per line.
+ *
+ *  @param table  Array of tab_entry in ascending string order.
+ */
+template<typename T>
+static void
+list_table(const T& table)
+{
+    for (auto& i : table) {
+	cout << i.s << '\n';
+    }
+}
+
 static void show_usage() {
     cout << "Usage: " PROG_NAME " [OPTIONS] 'QUERY'\n"
 "NB: QUERY should be quoted to protect it from the shell.\n\n"
@@ -179,7 +226,12 @@ static void show_usage() {
 "  -m, --msize=MSIZE                 maximum number of matches to return\n"
 "  -c, --check-at-least=HOWMANY      minimum number of matches to check\n"
 "  -s, --stemmer=LANG                set the stemming language, the default is\n"
-"                                    'english' (pass 'none' to disable stemming)\n"
+"                                    'english' (pass 'none' to disable stemming).\n"
+"                                    Valid stemmers:"
+<< print_stemmers() <<
+"  -S, --stem-strategy=STRATEGY      set the stemming strategy (default: some).\n"
+"                                    Valid strategies:"
+<< print_table(stem_strategy_tab) <<
 "  -p, --prefix=PFX:TERMPFX          add a prefix\n"
 "  -b, --boolean-prefix=PFX:TERMPFX  add a boolean prefix\n"
 "  -f, --flags=FLAG1[,FLAG2]...      specify QueryParser flags (default:\n"
@@ -199,12 +251,13 @@ static void show_usage() {
 int
 main(int argc, char **argv)
 try {
-    const char * opts = "d:m:c:s:p:b:f:o:w:Fhv";
+    const char * opts = "d:m:c:s:S:p:b:f:o:w:Fhv";
     static const struct option long_opts[] = {
 	{ "db",		required_argument, 0, 'd' },
 	{ "msize",	required_argument, 0, 'm' },
 	{ "check-at-least",	required_argument, 0, 'c' },
 	{ "stemmer",	required_argument, 0, 's' },
+	{ "stem-strategy",	required_argument, 0, 'S' },
 	{ "prefix",	required_argument, 0, 'p' },
 	{ "boolean-prefix",	required_argument, 0, 'b' },
 	{ "flags",	required_argument, 0, 'f' },
@@ -292,7 +345,9 @@ try {
 			*comma++ = '\0';
 		    int flag = decode(flag_tab, optarg);
 		    if (flag < 0) {
-			cerr << "Unknown flag '" << optarg << "'\n";
+			cerr << "Unknown flag '" << optarg << "'\n"
+				"Available flags are:\n";
+			list_table(flag_tab);
 			exit(1);
 		    }
 		    flags |= unsigned(flag);
@@ -302,12 +357,27 @@ try {
 	    case 'o': {
 		int op = decode(default_op_tab, optarg);
 		if (op < 0) {
-		    cerr << "Unknown op '" << optarg << "'\n";
+		    cerr << "Unknown operator '" << optarg << "'\n"
+			    "Available operators are:\n";
+		    list_table(default_op_tab);
 		    exit(1);
 		}
 		parser.set_default_op(static_cast<Xapian::Query::op>(op));
 		break;
 	    }
+	    case 'S': {
+		int s = decode(stem_strategy_tab, optarg);
+		if (s < 0) {
+		    cerr << "Unknown stem strategy '" << optarg << "'\n"
+			    "Available stem strategies are:\n";
+		    list_table(stem_strategy_tab);
+		    exit(1);
+		}
+		auto strategy =
+		    static_cast<Xapian::QueryParser::stem_strategy>(s);
+		parser.set_stemming_strategy(strategy);
+		break;
+	    }
 	    case 'w': {
 		weight = decode(wt_tab, optarg);
 		if (weight < 0) {
@@ -340,7 +410,6 @@ try {
 
     parser.set_database(db);
     parser.set_stemmer(stemmer);
-    parser.set_stemming_strategy(Xapian::QueryParser::STEM_SOME);
     parser.set_stopper(&mystopper);
 
     if (!flags_set) {
