@@ -1,9 +1,9 @@
-/** @file database.h
+/** @file
  * @brief API for working with Xapian databases
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2011,2012,2013,2014,2015,2016,2017,2019 Olly Betts
+ * Copyright 2003-2026 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@
 #define XAPIAN_INCLUDED_DATABASE_H
 
 #if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
-# error "Never use <xapian/database.h> directly; include <xapian.h> instead."
+# error Never use <xapian/database.h> directly; include <xapian.h> instead.
 #endif
 
 #include <iosfwd>
@@ -89,7 +89,12 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 */
 	void add_database(const Database & database);
 
-	/** Return number of shards in this Database object. */
+	/** Return number of shards in this Database object.
+	 *
+	 *  If you want the number of documents, see @ get_doccount().
+	 *
+	 *  @since Xapian 1.4.12
+	 */
 	size_t size() const {
 	    return internal.size();
 	}
@@ -159,7 +164,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  or to recover from a Xapian::DatabaseModifiedError.
 	 *
 	 *  Calling reopen() on a database which has been closed (with @a
-	 *  close()) will always raise a Xapian::DatabaseError.
+	 *  close()) will always raise a Xapian::DatabaseClosedError.
 	 *
 	 *  @return	true if the database might have been reopened (if false
 	 *		is returned, the database definitely hasn't been
@@ -179,7 +184,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *
 	 *  Closing a database cannot be undone - in particular, calling
 	 *  reopen() after close() will not reopen it, but will instead throw a
-	 *  Xapian::DatabaseError exception.
+	 *  Xapian::DatabaseClosedError exception.
 	 *
 	 *  Calling close() again on a database which has already been closed
 	 *  has no effect (and doesn't raise an exception).
@@ -192,8 +197,8 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *     been closed (this can only happen if all the required data is
 	 *     cached)
 	 *
-	 *   - raise a Xapian::DatabaseError exception indicating that the
-	 *     database is closed.
+	 *   - raise a Xapian::DatabaseClosedError exception indicating that
+	 *     the database is closed.
 	 *
 	 *  The reason for this behaviour is that otherwise we'd have to check
 	 *  that the database is still open on every method call on every
@@ -229,6 +234,8 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/** An iterator pointing to the start of the termlist
 	 *  for a given document.
 	 *
+	 *  The terms are returned in ascending string order (by byte value).
+	 *
 	 *  @param did	The document id of the document to iterate terms for.
 	 */
 	TermIterator termlist_begin(Xapian::docid did) const;
@@ -254,6 +261,8 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	}
 
 	/** An iterator which runs across all terms with a given prefix.
+	 *
+	 *  The terms are returned in ascending string order (by byte value).
 	 *
 	 *  @param prefix The prefix to restrict the returned terms to (default:
 	 *		  iterate all terms)
@@ -275,6 +284,14 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/// Get the average length of the documents in the database.
 	Xapian::doclength get_avlength() const;
 
+	/** New name for get_avlength().
+	 *
+	 *  Added for forward compatibility with the next release series.
+	 *
+	 *  @since 1.4.17.
+	 */
+	double get_average_length() const { return get_avlength(); }
+
 	/** Get the total length of all the documents in the database.
 	 *
 	 *  Added in Xapian 1.4.5.
@@ -604,7 +621,8 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *       space (the default).
 	 *     - Xapian::Compactor::FULLER   - Allow oversize items to save
 	 *       more space (not recommended if you ever plan to update the
-	 *       compacted database).
+	 *       compacted database).  @since 1.4.31 Has the same effect as
+	 *       FULL.
 	 *
 	 *  @param block_size This specifies the block size (in bytes) for
 	 *		to use for the output.  For glass, the block size must
@@ -817,7 +835,9 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  using bitwise-or (| in C++):
 	 *
 	 *   - Xapian::DB_NO_SYNC don't call fsync() or similar
+	 *   - Xapian::DB_FULL_SYNC try harder to ensure data is safe
 	 *   - Xapian::DB_DANGEROUS don't be crash-safe, no concurrent readers
+	 *   - Xapian::DB_NO_TERMLIST don't use a termlist table
 	 *   - Xapian::DB_RETRY_LOCK to wait to get a write lock
 	 *
 	 *  @param block_size If a new database is created, this specifies
@@ -870,6 +890,22 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	}
 #endif
 
+	/** Add shards from another WritableDatabase.
+	 *
+	 *  Any shards in @a other are added to the list of shards in this
+	 *  object.  The shards are reference counted and also remain in
+	 *  @a other.
+	 *
+	 *  @param other Another WritableDatabase object to add shards from
+	 */
+	void add_database(const WritableDatabase& other) {
+	    // This method is provided mainly so that adding a Database to a
+	    // WritableDatabase is a compile-time error - prior to 1.4.19, it
+	    // would essentially act as a "black-hole" shard which discarded
+	    // any changes made to it.
+	    Database::add_database(other);
+	}
+
 	/** Commit any pending modifications made to the database.
 	 *
 	 *  For efficiency reasons, when performing multiple updates to a
@@ -886,6 +922,10 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  performed or not performed at all: it is then up to the
 	 *  application to work out which operations need to be repeated.
 	 *
+	 *  However, note that if called on a sharded database, atomicity isn't
+	 *  guaranteed between shards - it's possible for the changes to one
+	 *  shard to be committed but changes to another shard to fail.
+	 *
 	 *  It's not valid to call commit() within a transaction.
 	 *
 	 *  Beware of calling commit() too frequently: this will make indexing
@@ -924,6 +964,10 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  a power failure, this characteristic should be preserved (as long
 	 *  as the filesystem isn't corrupted, etc).
 	 *
+	 *  However, note that if called on a sharded database, atomicity isn't
+	 *  guaranteed between shards.  Within each shard, the transaction will
+	 *  still act atomically.
+	 *
 	 *  A transaction is started with begin_transaction() and can
 	 *  either be committed by calling commit_transaction() or aborted
 	 *  by calling cancel_transaction().
@@ -977,6 +1021,10 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *
 	 *  In all cases the transaction will no longer be in progress.
 	 *
+	 *  Note that if called on a sharded database, atomicity isn't
+	 *  guaranteed between shards.  Within each shard, the transaction will
+	 *  still act atomically.
+	 *
 	 *  @exception Xapian::DatabaseError will be thrown if a problem occurs
 	 *             while modifying the database.
 	 *
