@@ -1,4 +1,4 @@
-/** @file dbfactory.h
+/** @file
  * @brief Factory functions for constructing Database and WritableDatabase objects
  */
 /* Copyright (C) 2005,2006,2007,2008,2009,2011,2013,2014,2016 Olly Betts
@@ -23,7 +23,7 @@
 #define XAPIAN_INCLUDED_DBFACTORY_H
 
 #if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
-# error "Never use <xapian/dbfactory.h> directly; include <xapian.h> instead."
+# error Never use <xapian/dbfactory.h> directly; include <xapian.h> instead.
 #endif
 
 #ifndef _MSC_VER
