@@ -1,4 +1,4 @@
-/** @file  eset.h
+/** @file
  *  @brief Class representing a list of query expansion terms
  */
 /* Copyright (C) 2015,2016 Olly Betts
@@ -23,7 +23,7 @@
 #define XAPIAN_INCLUDED_ESET_H
 
 #if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
-# error "Never use <xapian/eset.h> directly; include <xapian.h> instead."
+# error Never use <xapian/eset.h> directly; include <xapian.h> instead.
 #endif
 
 #include <iterator>
@@ -136,9 +136,9 @@ class XAPIAN_VISIBILITY_DEFAULT ESet {
     /// @private
     typedef const value_type * const_pointer;
     /// @private
-    typedef value_type & reference;
+    typedef value_type reference;
     /// @private
-    typedef const value_type & const_reference;
+    typedef const value_type const_reference;
     // @}
     //
     /** @private @internal ESet is what the C++ STL calls a container.
@@ -224,9 +224,9 @@ class XAPIAN_VISIBILITY_DEFAULT ESetIterator {
     /// @private
     typedef Xapian::termcount_diff difference_type;
     /// @private
-    typedef std::string * pointer;
+    typedef value_type* pointer;
     /// @private
-    typedef std::string & reference;
+    typedef value_type reference;
     // @}
 
     /// Move the iterator forwards by n positions.
