@@ -1,7 +1,7 @@
-/** @file  mset.h
+/** @file
  *  @brief Class representing a list of search results
  */
-/* Copyright (C) 2015,2016 Olly Betts
+/* Copyright (C) 2015,2016,2019,2023 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +23,7 @@
 #define XAPIAN_INCLUDED_MSET_H
 
 #if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
-# error "Never use <xapian/mset.h> directly; include <xapian.h> instead."
+# error Never use <xapian/mset.h> directly; include <xapian.h> instead.
 #endif
 
 #include <iterator>
@@ -65,6 +65,14 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
      */
     MSet & operator=(const MSet & o);
 
+#ifdef XAPIAN_MOVE_SEMANTICS
+    /// Move constructor.
+    MSet(MSet && o);
+
+    /// Move assignment operator.
+    MSet & operator=(MSet && o);
+#endif
+
     /** Default constructor.
      *
      *  Creates an empty MSet, mostly useful as a placeholder.
@@ -102,7 +110,11 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 
     /** Get the termfreq of a term.
      *
-     *  @return The number of documents @a term occurs in.
+     *  @return The number of documents which @a term occurs in.  This
+     *		considers all documents in the database being searched, so
+     *		gives the same answer as <code>db.get_termfreq(term)</code>
+     *		(but is more efficient for query terms as it returns a
+     *		value cached during the search.)
      */
     Xapian::doccount get_termfreq(const std::string & term) const;
 
@@ -171,24 +183,67 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	 *  was found in text. If not enabled, snippet() returns a (sub)string
 	 *  of text without any highlighted terms.
 	 */
-	SNIPPET_EMPTY_WITHOUT_MATCH = 4
+	SNIPPET_EMPTY_WITHOUT_MATCH = 4,
+
+	/** Generate n-grams for scripts without explicit word breaks.
+	 *
+	 *  Text in other scripts is split into words as normal.
+	 *
+	 *  Enable this option to highlight search results for queries parsed
+	 *  with the QueryParser::FLAG_NGRAMS flag.
+	 *
+	 *  The TermGenerator::FLAG_NGRAMS flag needs to have been used at
+	 *  index time.
+	 *
+	 *  This mode can also be enabled by setting environment variable
+	 *  XAPIAN_CJK_NGRAM to a non-empty value (but doing so was deprecated
+	 *  in 1.4.11).
+	 *
+	 *  In 1.4.x this feature was specific to CJK (Chinese, Japanese and
+	 *  Korean), but in 2.0.0 it's been extended to other languages.  To
+	 *  reflect this change the new and preferred name is SNIPPET_NGRAMS,
+	 *  which was added as an alias for forward compatibility in Xapian
+	 *  1.4.23.  Use SNIPPET_CJK_NGRAM instead if you aim to support Xapian
+	 *  &lt; 1.4.23.
+	 *
+	 *  @since Added in Xapian 1.4.23.
+	 */
+	SNIPPET_NGRAMS = 2048,
+
+	/** Generate n-grams for scripts without explicit word breaks.
+	 *
+	 *  Old name - use SNIPPET_NGRAMS instead unless you aim to support
+	 *  Xapian &lt; 1.4.23.
+	 *
+	 *  @since Added in Xapian 1.4.11.
+	 */
+	SNIPPET_CJK_NGRAM = SNIPPET_NGRAMS
     };
 
     /** Generate a snippet.
      *
-     *  This method selects a continuous run of words of less than about @a
-     *  length bytes from @a text, based mainly on where the query matches
-     *  (currently terms, exact phrases and wildcards are taken into account),
-     *  but also on the non-query terms in the text.
+     *  This method selects a continuous run of words from @a text, based
+     *  mainly on where the query matches (currently terms, exact phrases and
+     *  wildcards are taken into account).  If flag SNIPPET_BACKGROUND_MODEL is
+     *  used (which it is by default) then the selection algorithm also
+     *  considers the non-query terms in the text with the aim of showing
+     *  a context which provides more useful information.
      *
-     *  The returned text can be escaped (by default, it is escaped to make it
-     *  suitable for use in HTML), and matches with the query will be
+     *  The size of the text selected can be controlled by the @a length
+     *  parameter, which specifies a number of bytes of text to aim to select.
+     *  However slightly more text may be selected.  Also the size of any
+     *  escaping, highlighting or omission markers is not considered.
+     *
+     *  The returned text is escaped to make it suitable for use in HTML
+     *  (though beware that in upstream releases 1.4.5 and earlier this
+     *  escaping was sometimes incomplete), and matches with the query will be
      *  highlighted using @a hi_start and @a hi_end.
      *
      *  If the snippet seems to start or end mid-sentence, then @a omit is
      *  prepended or append (respectively) to indicate this.
      *
-     *  The stemmer used to build the query should be specified in @a stemmer.
+     *  The same stemming algorithm which was used to build the query should be
+     *  specified in @a stemmer.
      *
      *  And @a flags contains flags controlling behaviour.
      *
@@ -289,9 +344,9 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
     /// @private
     typedef const value_type * const_pointer;
     /// @private
-    typedef value_type & reference;
+    typedef value_type reference;
     /// @private
-    typedef const value_type & const_reference;
+    typedef const value_type const_reference;
     // @}
     //
     /** @private @internal MSet is what the C++ STL calls a container.
@@ -373,13 +428,13 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
     /// @private
     typedef std::random_access_iterator_tag iterator_category;
     /// @private
-    typedef std::string value_type;
+    typedef Xapian::docid value_type;
     /// @private
     typedef Xapian::termcount_diff difference_type;
     /// @private
-    typedef std::string * pointer;
+    typedef value_type* pointer;
     /// @private
-    typedef std::string & reference;
+    typedef value_type reference;
     // @}
 
     /// Move the iterator forwards by n positions.
@@ -454,6 +509,14 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
      */
     Xapian::doccount get_collapse_count() const;
 
+    /** Return the sort key for the current position.
+     *
+     *  If sorting didn't use a key then an empty string will be returned.
+     *
+     *  @since Added in Xapian 1.4.6.
+     */
+    std::string get_sort_key() const;
+
     /** Convert the weight of the current iterator position to a percentage.
      *
      *  The matching document with the highest weight will get 100% if it
