@@ -78,7 +78,7 @@ class XAPIAN_VISIBILITY_DEFAULT PostingIterator {
      *  assigned to, but is sometimes syntactically convenient.
      */
     XAPIAN_NOTHROW(PostingIterator())
-	: internal(0) { }
+	: internal() { }
 
     /// Destructor.
     ~PostingIterator() {
@@ -149,9 +149,9 @@ class XAPIAN_VISIBILITY_DEFAULT PostingIterator {
     /// @private
     typedef Xapian::doccount_diff difference_type;
     /// @private
-    typedef Xapian::docid * pointer;
+    typedef value_type* pointer;
     /// @private
-    typedef Xapian::docid & reference;
+    typedef value_type reference;
     // @}
 
   private:
