@@ -1,7 +1,7 @@
 /** @file
  * @brief Xapian::Query API class
  */
-/* Copyright (C) 2011,2012,2013,2014,2015,2016,2017,2018,2019 Olly Betts
+/* Copyright (C) 2011-2025 Olly Betts
  * Copyright (C) 2008 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -42,6 +42,16 @@ namespace Xapian {
 
 class PostingSource;
 
+namespace Internal {
+    // You need a C++11 compiler to build Xapian 1.4.x, but we aim to support
+    // the headers being included without having to select C++11 mode.  That's
+    // become much less of an issue as most compilers have switched to at least
+    // C++11 by default, but it seems better not to change this mid release
+    // series, so we define our own version of std::enable_if<>.
+    template<bool, typename T = void> struct EnableIf { };
+    template<typename T> struct EnableIf<true, T> { typedef T type; };
+}
+
 /// Class representing a query.
 class XAPIAN_VISIBILITY_DEFAULT Query {
   public:
@@ -184,13 +194,15 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 *  large OR query, but it doesn't matter if the search completely
 	 *  ignores some of the less important terms in the query.
 	 *
-	 *  The subqueries don't have to be terms, but if they aren't then
-	 *  OP_ELITE_SET will look at the estimated frequencies of the
-	 *  subqueries and so could pick a subset which don't actually
-	 *  match any documents even if the full OR would match some.
+	 *  The subqueries don't have to be terms.  If they aren't then
+	 *  OP_ELITE_SET could potentially pick a subset which doesn't
+	 *  actually match any documents even if the full OR would match some
+	 *  (because OP_ELITE_SET currently selects those subqueries which can
+	 *  return the highest weights).  This is probably rare in practice
+	 *  though.
 	 *
-	 *  You can specify a parameter to the query constructor which control
-	 *  the number of terms which OP_ELITE_SET will pick.  If not
+	 *  You can specify a parameter to the query constructor which controls
+	 *  the number of subqueries which OP_ELITE_SET will pick.  If not
 	 *  specified, this defaults to 10 (Xapian used to default to
 	 *  <code>ceil(sqrt(number_of_subqueries))</code> if there are more
 	 *  than 100 subqueries, but this rather arbitrary special case was
@@ -200,8 +212,15 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 *  Xapian::Query query(Xapian::Query::OP_ELITE_SET, subqs.begin(), subqs.end(), 7);
 	 *  </pre>
 	 *
-	 * If the number of subqueries is less than this threshold,
-	 * OP_ELITE_SET behaves identically to OP_OR.
+	 *  If the number of subqueries is less than this threshold,
+	 *  OP_ELITE_SET behaves identically to OP_OR.
+	 *
+	 *  When used with a sharded database, OP_ELITE_SET currently picks
+	 *  the subqueries to use separately for each shard based on the
+	 *  maximum weight they can return in that shard.  This means it
+	 *  probably won't select exactly the same terms, and so the results
+	 *  of the search may not be exactly the same as for a single database
+	 *  with equivalent contents.
 	 */
 	OP_ELITE_SET = 10,
 
@@ -454,11 +473,14 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *  @param window	Window size for OP_NEAR and OP_PHRASE, or 0 to use the
      *			number of subqueries as the window size (default: 0).
      */
-    template<typename I>
+    template<typename I,
+	typename T = typename std::iterator_traits<I>,
+	typename V = typename T::value_type,
+	typename Xapian::Internal::EnableIf<sizeof(V) != 1, bool>::type = true,
+	typename iterator_category = typename T::iterator_category>
     Query(op op_, I begin, I end, Xapian::termcount window = 0)
     {
 	if (begin != end) {
-	    typedef typename std::iterator_traits<I>::iterator_category iterator_category;
 	    init(op_, window, begin, end, iterator_category());
 	    bool positional = (op_ == OP_NEAR || op_ == OP_PHRASE);
 	    for (I i = begin; i != end; ++i) {
