@@ -1,7 +1,7 @@
-/** @file queryparser.h
+/** @file
  * @brief parsing a user query string to build a Xapian::Query object
  */
-/* Copyright (C) 2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Olly Betts
+/* Copyright (C) 2005-2023 Olly Betts
  * Copyright (C) 2010 Adam Sjøgren
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
 #define XAPIAN_INCLUDED_QUERYPARSER_H
 
 #if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
-# error "Never use <xapian/queryparser.h> directly; include <xapian.h> instead."
+# error Never use <xapian/queryparser.h> directly; include <xapian.h> instead.
 #endif
 
 #include <xapian/attributes.h>
@@ -42,7 +42,11 @@ namespace Xapian {
 class Database;
 class Stem;
 
-/// Base class for stop-word decision functor.
+/** Abstract base class for stop-word decision functor.
+ *
+ *  If you just want to use an existing stopword list, see
+ *  Xapian::SimpleStopper.
+ */
 class XAPIAN_VISIBILITY_DEFAULT Stopper
     : public Xapian::Internal::opt_intrusive_base {
     /// Don't allow assignment.
@@ -69,7 +73,7 @@ class XAPIAN_VISIBILITY_DEFAULT Stopper
 
     /** Start reference counting this object.
      *
-     *  You can hand ownership of a dynamically allocated Stopper
+     *  You can transfer ownership of a dynamically allocated Stopper
      *  object to Xapian by calling release() and then passing the object to a
      *  Xapian method.  Xapian will arrange to delete the object once it is no
      *  longer required.
@@ -81,7 +85,7 @@ class XAPIAN_VISIBILITY_DEFAULT Stopper
 
     /** Start reference counting this object.
      *
-     *  You can hand ownership of a dynamically allocated Stopper
+     *  You can transfer ownership of a dynamically allocated Stopper
      *  object to Xapian by calling release() and then passing the object to a
      *  Xapian method.  Xapian will arrange to delete the object once it is no
      *  longer required.
@@ -102,12 +106,16 @@ class XAPIAN_VISIBILITY_DEFAULT SimpleStopper : public Stopper {
 
     /** Initialise from a pair of iterators.
      *
-     * Xapian includes stop list files for many languages. You can initialise from a file like that:
-     * @code
-     * ifstream words("stopwords/english/stop.txt");
-     * Xapian::SimplerStopper stopper(istream_iterator<string>(words), istream_iterator<string>());
-     * @endcode
+     *  Xapian includes stopword list files for many languages.  You can
+     *  initialise from a file like so:
+     *  @code
+     *  std::ifstream words("stopwords/english/stop.txt");
+     *  Xapian::SimplerStopper stopper(std::istream_iterator<std::string>(words), std::istream_iterator<std::string>());
+     *  @endcode
      *
+     *  In bindings for other languages it isn't possible to pass a C++
+     *  iterator pair, so instead this constructor is wrapped to allow
+     *  passing a filename.
      */
     template<class Iterator>
     SimpleStopper(Iterator begin, Iterator end) : stop_words(begin, end) { }
@@ -175,10 +183,12 @@ class XAPIAN_VISIBILITY_DEFAULT RangeProcessor
      *			 * Xapian::RP_SUFFIX - require @a str_ as a suffix
      *			   instead of a prefix.
      *			 * Xapian::RP_REPEATED - optionally allow @a str_
-     *			   on both ends of the range - e.g. $1..$10 or
-     *			   5m..50m.  By default a prefix is only checked for on
-     *			   the start (e.g. date:1/1/1980..31/12/1989), and a
-     *			   suffix only on the end (e.g. 2..12kg).
+     *			   on both ends of the range, or only on the non-empty
+     *			   end - e.g. `$1..$10`, `..$10`, `5m..50m`, or `5m..`.
+     *			   By default a prefix is only checked for on
+     *			   the start (e.g. `date:1/1/1980..31/12/1989` or
+     *			   `date:..31/12/1989`), and a suffix only on the end
+     *			   (e.g. `2..12kg` or `2..kg`).
      */
     explicit RangeProcessor(Xapian::valueno slot_,
 			    const std::string& str_ = std::string(),
@@ -215,7 +225,7 @@ class XAPIAN_VISIBILITY_DEFAULT RangeProcessor
 
     /** Start reference counting this object.
      *
-     *  You can hand ownership of a dynamically allocated RangeProcessor
+     *  You can transfer ownership of a dynamically allocated RangeProcessor
      *  object to Xapian by calling release() and then passing the object to a
      *  Xapian method.  Xapian will arrange to delete the object once it is no
      *  longer required.
@@ -227,7 +237,7 @@ class XAPIAN_VISIBILITY_DEFAULT RangeProcessor
 
     /** Start reference counting this object.
      *
-     *  You can hand ownership of a dynamically allocated RangeProcessor
+     *  You can transfer ownership of a dynamically allocated RangeProcessor
      *  object to Xapian by calling release() and then passing the object to a
      *  Xapian method.  Xapian will arrange to delete the object once it is no
      *  longer required.
@@ -269,8 +279,9 @@ class XAPIAN_VISIBILITY_DEFAULT DateRangeProcessor : public RangeProcessor {
      *
      *  @param slot_	The value slot number to query.
      *
-     *  @param str_	A string to look for to recognise values as belonging
-     *			to this date range.
+     *  @param str_	A prefix or suffix string to look for to recognise
+     *			values as belonging to this date range (@a flags_
+     *			determines whether this is a prefix or suffix).
      *
      *  @param flags_	Zero or more of the following flags, combined with
      *			bitwise-or:
@@ -295,16 +306,9 @@ class XAPIAN_VISIBILITY_DEFAULT DateRangeProcessor : public RangeProcessor {
      *  if prefix_ is false, the second value in a range must end with str_
      *  (and the first value may optionally end with str_).
      *
-     *  If str_ is empty, the Xapian::RP_SUFFIX and Xapian::RP_REPEATED are
-     *  irrelevant, and no special strings are required at the start or end of
-     *  the strings defining the range.
-     *
-     *  The remainder of both strings defining the endpoints must be valid
-     *  dates.
-     *
-     *  For example, if str_ is "created:", Xapian::RP_SUFFIX is not specified,
-     *  and the range processor has been added to the queryparser, the
-     *  queryparser will accept "created:1/1/2000..31/12/2001".
+     *  If str_ is empty then no prefix or suffix is checked for (and
+     *  Xapian::RP_SUFFIX and Xapian::RP_REPEATED are irrelevant) - anything
+     *  which looks like a date range will be processed.
      */
     DateRangeProcessor(Xapian::valueno slot_, const std::string &str_,
 		       unsigned flags_ = 0, int epoch_year_ = 1970)
@@ -312,15 +316,38 @@ class XAPIAN_VISIBILITY_DEFAULT DateRangeProcessor : public RangeProcessor {
 	  epoch_year(epoch_year_) { }
 
     /** Check for a valid date range.
-     *
-     *  If any specified prefix is present, and the range looks like a
-     *  date range, the dates are converted to the format YYYYMMDD and
-     *  combined into a value range query.
      *
      *  @param begin	The start of the range as specified in the query string
      *			by the user.
      *  @param end	The end of the range as specified in the query string
      *			by the user.
+     *
+     *  If a prefix or suffix was specified at construction time, that must
+     *  be present on @a begin and/or @a end (taking @a Xapian::RP_SUFFIX and
+     *  @a Xapian::RP_REPEATED into account).
+     *
+     *  Then if the range looks like a date range, the dates are converted to
+     *  the format YYYYMMDD and combined into a value range query.
+     *
+     *  Most numeric date formats are recognised:
+     *
+     *  * YYYYMMDD - e.g. `20251225..20260214`
+     *  * YYYY-MM-DD (ISO) - e.g. `2025-12-25..2026-02-14`
+     *  * month/day/year (US) - e.g. `12/25/2025..2/14/2026`
+     *  * day/month/year (much of the world) - e.g. `25/12/2025..14/2/2026`
+     *
+     *  The delimiter can be `-`, `/` or `.` for any of the delimited formats,
+     *  but must be the same within a single date.
+     *
+     *  Some dates are unfortunately ambiguous and could be in either of
+     *  the last two formats - for example `2/3/2025` could be March 2nd or
+     *  February 3rd.  The range start is assumed to be before the range
+     *  end which can resolve some ambiguous cases, but otherwise
+     *  day/month/year is assumed unless flag @a Xapian::RP_DATE_PREFER_MDY
+     *  was specified.
+     *
+     *  Two digit years can be used with the last two formats - constructor
+     *  parameter @a epoch_year_ determines how these are interpreted.
      */
     Xapian::Query operator()(const std::string& begin, const std::string& end);
 };
@@ -743,7 +770,7 @@ class XAPIAN_VISIBILITY_DEFAULT FieldProcessor
 
     /** Start reference counting this object.
      *
-     *  You can hand ownership of a dynamically allocated FieldProcessor
+     *  You can transfer ownership of a dynamically allocated FieldProcessor
      *  object to Xapian by calling release() and then passing the object to a
      *  Xapian method.  Xapian will arrange to delete the object once it is no
      *  longer required.
@@ -755,7 +782,7 @@ class XAPIAN_VISIBILITY_DEFAULT FieldProcessor
 
     /** Start reference counting this object.
      *
-     *  You can hand ownership of a dynamically allocated FieldProcessor
+     *  You can transfer ownership of a dynamically allocated FieldProcessor
      *  object to Xapian by calling release() and then passing the object to a
      *  Xapian method.  Xapian will arrange to delete the object once it is no
      *  longer required.
@@ -862,20 +889,64 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	 */
 	FLAG_AUTO_MULTIWORD_SYNONYMS = 1024,
 
-	/** Enable generation of n-grams from CJK text.
+	/** Generate n-grams for scripts without explicit word breaks.
 	 *
-	 *  With this enabled, spans of CJK characters are split into unigrams
+	 *  Spans of characters in such scripts are split into unigrams
 	 *  and bigrams, with the unigrams carrying positional information.
-	 *  Non-CJK characters are split into words as normal.
+	 *  Text in other scripts is split into words as normal.
 	 *
-	 *  The corresponding option needs to have been used at index time.
+	 *  The TermGenerator::FLAG_NGRAMS flag needs to have been used at
+	 *  index time.
 	 *
-	 *  Flag added in Xapian 1.3.4 and 1.2.22.  This mode can be
-	 *  enabled in 1.2.8 and later by setting environment variable
-	 *  XAPIAN_CJK_NGRAM to a non-empty value (but doing so was deprecated
-	 *  in 1.4.11).
+	 *  This mode can also be enabled in 1.2.8 and later by setting
+	 *  environment variable XAPIAN_CJK_NGRAM to a non-empty value (but
+	 *  doing so was deprecated in 1.4.11).
+	 *
+	 *  In 1.4.x this feature was specific to CJK (Chinese, Japanese and
+	 *  Korean), but in 2.0.0 it's been extended to other languages.  To
+	 *  reflect this change the new and preferred name is FLAG_NGRAMS,
+	 *  which was added as an alias for forward compatibility in Xapian
+	 *  1.4.23.  Use FLAG_CJK_NGRAM instead if you aim to support Xapian
+	 *  &lt; 1.4.23.
+	 *
+	 *  @since Added in Xapian 1.4.23.
 	 */
-	FLAG_CJK_NGRAM = 2048,
+	FLAG_NGRAMS = 2048,
+
+	/** Generate n-grams for scripts without explicit word breaks.
+	 *
+	 *  Old name - use FLAG_NGRAMS instead unless you aim to support Xapian
+	 *  &lt; 1.4.23.
+	 *
+	 *  @since Added in Xapian 1.3.4 and 1.2.22.
+	 */
+	FLAG_CJK_NGRAM = FLAG_NGRAMS,
+
+	/** Accumulate unstem and stoplist results.
+	 *
+	 *  By default, the unstem and stoplist data is reset by a call to
+	 *  parse_query(), which makes sense if you use the same QueryParser
+	 *  object to parse a series of independent queries.
+	 *
+	 *  If you're using the same QueryParser object to parse several
+	 *  fields on the same query form, you may want to have the unstem
+	 *  and stoplist data combined for all of them, in which case you
+	 *  can use this flag to prevent this data from being reset.
+	 *
+	 *  @since Added in Xapian 1.4.18.
+	 */
+	FLAG_ACCUMULATE = 65536,
+
+	/** Produce a query which doesn't use positional information.
+	 *
+	 *  With this flag enabled, no positional information will be used
+	 *  and any query operations which would use it are replaced by
+	 *  the nearest equivalent which doesn't (so phrase searches, NEAR
+	 *  and ADJ will result in OP_AND).
+	 *
+	 *  @since Added in Xapian 1.4.19.
+	 */
+	FLAG_NO_POSITIONS = 0x20000,
 
 	/** The default flags.
 	 *
@@ -1092,8 +1163,13 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *  In 1.0.3 and earlier, subsequent calls to this method with the same
      *  value of @a field had no effect.
      *
-     *  @param field   The user visible field name
-     *  @param prefix  The term prefix to map this to
+     *  @param field   The user visible field name.  Currently this needs to
+     *		       consist of characters for which
+     *		       Xapian::Unicode::is_wordchar() is true (approximately
+     *		       alphanumerics plus connector punctuation such as `_`).
+     *		       Since 1.4.26 it can optionally end in a `:` for
+     *		       consistency with how range prefixes are specified.
+     *  @param prefix  The term prefix to map this to.
      */
     void add_prefix(const std::string& field, const std::string& prefix);
 
@@ -1143,7 +1219,13 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *  In 1.0.3 and earlier, subsequent calls to this method with the same
      *  value of @a field had no effect.
      *
-     *  @param field   The user visible field name
+     *  @param field   The user visible field name, which may not be empty
+     *		       for a boolean filter.  Currently this needs to
+     *		       consist of characters for which
+     *		       Xapian::Unicode::is_wordchar() is true (approximately
+     *		       alphanumerics plus connector punctuation such as `_`).
+     *		       Since 1.4.26 it can optionally end in a `:` for
+     *		       consistency with how range prefixes are specified.
      *  @param prefix  The term prefix to map this to
      *  @param grouping	Controls how multiple filters are combined - filters
      *			with the same grouping value are combined with OP_OR,
@@ -1163,7 +1245,13 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *  This is an older version of this method - use the version with
      *  the `grouping` parameter in preference to this one.
      *
-     *  @param field   The user visible field name
+     *  @param field   The user visible field name, which may not be empty
+     *		       for a boolean filter.  Currently this needs to
+     *		       consist of characters for which
+     *		       Xapian::Unicode::is_wordchar() is true (approximately
+     *		       alphanumerics plus connector punctuation such as `_`).
+     *		       Since 1.4.26 it can optionally end in a `:` for
+     *		       consistency with how range prefixes are specified.
      *  @param prefix  The term prefix to map this to
      *  @param exclusive Controls how multiple filters are combined.  If
      *			true then @a prefix is used as the `grouping` value,
@@ -1229,6 +1317,11 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *  deprecated - use @a add_rangeprocessor() with a RangeProcessor instead.
      */
     XAPIAN_DEPRECATED(void add_valuerangeprocessor(Xapian::ValueRangeProcessor * vrproc)) {
+#ifdef __GNUC__
+// Avoid deprecation warnings if compiling without optimisation.
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 	/// Compatibility shim.
 	class ShimRangeProcessor : public RangeProcessor {
 	    Xapian::Internal::opt_intrusive_ptr<Xapian::ValueRangeProcessor> vrp;
@@ -1249,6 +1342,9 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	};
 
 	add_rangeprocessor((new ShimRangeProcessor(vrproc))->release());
+#ifdef __GNUC__
+# pragma GCC diagnostic pop
+#endif
     }
 
     /** Get the spelling-corrected query string.
