@@ -1,4 +1,4 @@
-/** @file stem.h
+/** @file
  * @brief stemming algorithms
  */
 /* Copyright (C) 2005,2007,2010,2011,2013,2014,2015,2018,2019 Olly Betts
@@ -23,7 +23,7 @@
 #define XAPIAN_INCLUDED_STEM_H
 
 #if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
-# error "Never use <xapian/stem.h> directly; include <xapian.h> instead."
+# error Never use <xapian/stem.h> directly; include <xapian.h> instead.
 #endif
 
 #include <xapian/constinfo.h>
@@ -102,13 +102,12 @@ class XAPIAN_VISIBILITY_DEFAULT Stem {
      *  - english (en) - Martin Porter's 2002 revision of his stemmer
      *  - earlyenglish - Early English (e.g. Shakespeare, Dickens) stemmer
      *    (since Xapian 1.3.2)
-     *  - english_lovins (lovins) - Lovin's stemmer
-     *  - english_porter (porter) - Porter's stemmer as described in
-     *			his 1980 paper
+     *  - lovins - Lovin's English stemmer
+     *  - porter - Porter's English stemmer exactly matching his 1980 paper
      *  - finnish (fi)
      *  - french (fr)
      *  - german (de)
-     *  - german2 - Normalises umlauts and &szlig;
+     *  - german2 - Normalises umlauts (e.g. ä and ae)
      *  - hungarian (hu)
      *  - indonesian (id) - Since Xapian 1.4.6
      *  - irish (ga) - Since Xapian 1.4.7
@@ -125,6 +124,11 @@ class XAPIAN_VISIBILITY_DEFAULT Stem {
      *  - tamil (ta) - Since Xapian 1.4.7
      *  - turkish (tr)
      *
+     *  Also dutch_porter is an alias for dutch since Xapian 1.4.28 (to
+     *  provide forward compatibility to the next release series where
+     *  we will switch dutch to refer to a version of the kraaij_pohlmann
+     *  stemmer).
+     *
      *  @param fallback If true then treat unknown @a language as "none",
      *			otherwise an exception is thrown (default: false).
      *			Parameter added in Xapian 1.4.14 - older versions
