@@ -170,7 +170,23 @@ const char * dummy[] = {
 "/// #endif",
 "/// @endcode",
 "///",
-"/// Added in Xapian 1.4.2.",
+"/// XAPIAN_AT_LEAST was added in Xapian 1.4.2.  This was released 2016-12-26",
+"/// so is now a reasonable hard requirement.",
+"///",
+"/// If you really want to support older versions beware that a check like:",
+"///",
+"/// #if defined XAPIAN_AT_LEAST && XAPIAN_AT_LEAST(1,4,12)",
+"///",
+"/// works with newer Xapian but fails to compile if the macro is not",
+"/// defined:",
+"///",
+"/// error: missing binary operator before token ‘(’",
+"///",
+"/// In this situation we recommend copying the definition of XAPIAN_AT_LEAST",
+"/// into your code and wrapping it in `#ifndef XAPIAN_AT_LEAST` ... `#endif`",
+"/// so you can use it unconditionally.",
+"///",
+"/// @since Xapian 1.4.2.",
 "#define XAPIAN_AT_LEAST(A,B,C) \\",
 "    (XAPIAN_MAJOR_VERSION > (A) || \\",
 "     (XAPIAN_MAJOR_VERSION == (A) && \\",
