@@ -510,7 +510,7 @@ DEFINE_TESTCASE(replacedoc8, writable) {
 
 /** Check that replacing a document after clear_terms() still deletes old
  *  positional data.  Regression test for bug introduced and fixed in
- *  development prior to 1.5.0.
+ *  development prior to 2.0.0.
  */
 DEFINE_TESTCASE(replacedoc9, writable) {
     Xapian::WritableDatabase db(get_named_writable_database("replacedoc9"));
@@ -1534,7 +1534,7 @@ DEFINE_TESTCASE(getrevision1, chert || glass) {
     TEST_EQUAL(db.get_revision(), 2);
 }
 
-/// Check get_revision() on an empty database reports 0.  (Since 1.5.0)
+/// Check get_revision() on an empty database reports 0.  (Since 2.0.0)
 DEFINE_TESTCASE(getrevision2, !backend) {
     Xapian::Database db;
     TEST_EQUAL(db.get_revision(), 0);
@@ -1644,7 +1644,7 @@ DEFINE_TESTCASE(nopositionbug1, backend) {
 
 /** Regression test for bug with get_mset(0, 0, N) (N > 0).
  *
- *  Fixed in 1.5.0 and 1.4.6.
+ *  Fixed in 1.4.6.
  */
 DEFINE_TESTCASE(checkatleast4, backend) {
     Xapian::Database db = get_database("apitest_simpledata");
@@ -1655,7 +1655,7 @@ DEFINE_TESTCASE(checkatleast4, backend) {
     TEST_EQUAL(mset.size(), 0);
 }
 
-/// Regression test for glass freelist leak fixed in 1.4.6 and 1.5.0.
+/// Regression test for glass freelist leak fixed in 1.4.6.
 DEFINE_TESTCASE(freelistleak1, check) {
     auto path = get_database_path("freelistleak1",
 				  [](Xapian::WritableDatabase& wdb,
@@ -1708,7 +1708,7 @@ DEFINE_TESTCASE(multidb1, backend) {
     db.add_database(db2);
     TEST_EQUAL(db.size(), db2.size());
     db.add_database(db2);
-    // Regression test for bug introduced and fixed in git master before 1.5.0.
+    // Regression test for bug introduced and fixed in git master before 2.0.0.
     // Adding a multi database to an empty database incorrectly worked just
     // like assigning the database object.  The list of shards is now copied
     // instead.
@@ -1799,9 +1799,9 @@ DEFINE_TESTCASE(unsupportedcheck3, !backend) {
 // Regression test for #824, fixed in 1.4.25.
 DEFINE_TESTCASE(corruptglass1, glass) {
     string db_path =
-	test_driver::get_srcdir() + "/testdata/glass_corrupt_db1";
+	test_driver::get_srcdir() + "/testdata/glass_corrupt_dbX";
 
-    for (int n = '1'; n <= '3'; ++n) {
+    for (int n = '1'; n <= '4'; ++n) {
 	db_path.back() = n;
 	TEST_EXCEPTION(Xapian::DatabaseCorruptError,
 		       Xapian::Database db(db_path));
