@@ -2,7 +2,7 @@
  * @brief tests which don't need a backend
  */
 /* Copyright (C) 2009 Richard Boulton
- * Copyright (C) 2009,2010,2011,2013,2014,2015,2016,2017,2018 Olly Betts
+ * Copyright (C) 2009-2026 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,6 +32,8 @@
 #include "testsuite.h"
 #include "testutils.h"
 
+#include <vector>
+
 using namespace std;
 
 // Check the version functions give consistent results.
@@ -241,6 +243,43 @@ DEFINE_TESTCASE(singlesubquery3, !backend) {
     singlesubquery3_(OP_MAX);
 }
 
+DEFINE_TESTCASE(pairwisequery1, !backend) {
+    // Test that constructing from char* and const char* gives the same result
+    // as explicitly constructing Query objects for the subqueries.  This
+    // serves as a regression test for using char*/const char* not compiling
+    // with Xapian <= 1.4.30.
+#define lit_a "a"
+#define lit_b "b"
+    char aa[] = lit_a;
+    char ab[] = lit_b;
+    const char* ca = aa;
+    const char* cb = ab;
+    Xapian::Query qa(aa);
+    Xapian::Query qb(ab);
+
+#define pairwisequery1_(OP) do {\
+	auto expect = Xapian::Query(qa.OP, lit_a, lit_b).get_description(); \
+	TEST_STRINGS_EQUAL(expect, \
+			   Xapian::Query(qa.OP, aa, ab).get_description()); \
+	TEST_STRINGS_EQUAL(expect, \
+			   Xapian::Query(qa.OP, ca, cb).get_description()); \
+	TEST_STRINGS_EQUAL(expect, \
+			   Xapian::Query(qa.OP, qa, qb).get_description()); \
+    } while (false)
+
+    pairwisequery1_(OP_AND);
+    pairwisequery1_(OP_OR);
+    pairwisequery1_(OP_AND_NOT);
+    pairwisequery1_(OP_XOR);
+    pairwisequery1_(OP_AND_MAYBE);
+    pairwisequery1_(OP_FILTER);
+    pairwisequery1_(OP_NEAR);
+    pairwisequery1_(OP_PHRASE);
+    pairwisequery1_(OP_ELITE_SET);
+    pairwisequery1_(OP_SYNONYM);
+    pairwisequery1_(OP_MAX);
+}
+
 /// Check we no longer combine wqf for same term at the same position.
 DEFINE_TESTCASE(combinewqfnomore1, !backend) {
     Xapian::Query q(Xapian::Query::OP_OR,
@@ -893,7 +932,7 @@ DEFINE_TESTCASE(nonutf8docdesc1, !backend) {
 
 /** Test removal of terms from a document while iterating over them.
  *
- *  Prior to 1.5.0 and 1.4.6 the underlying iterator was invalidated when
+ *  Prior to 1.4.6 the underlying iterator was invalidated when
  *  preinc == false, leading to undefined behaviour (typically a segmentation
  *  fault).
  */
@@ -1003,3 +1042,68 @@ DEFINE_TESTCASE(errorcopyctor, !backend) {
     }
     FAIL_TEST("Expected exception to be thrown");
 }
+
+// Test ESetIterator iterator_traits.
+DEFINE_TESTCASE(stlesetiterator, !backend) {
+    Xapian::ESet eset;
+    vector<string> v;
+    // This gave a compile error with stdc++ and -DGLIBCXX_DEBUG in 1.4.30:
+    v.insert(v.begin(), eset.begin(), eset.end());
+}
+
+// Test MSetIterator iterator_traits.
+DEFINE_TESTCASE(stlmsetiterator, !backend) {
+    Xapian::MSet mset;
+    vector<Xapian::docid> v;
+    // In Xapian <= 1.4.30 this gave a compile error with libc++, or
+    // with stdc++ and -DGLIBCXX_DEBUG:
+    v.insert(v.begin(), mset.begin(), mset.end());
+}
+
+// Test PositionIterator iterator_traits.
+DEFINE_TESTCASE(stlpositioniterator, !backend) {
+    Xapian::Database db;
+    vector<Xapian::termpos> v;
+    if (db.get_doccount() > 0) {
+	// In Xapian <= 1.4.30 this gave a compile error with stdc++ and
+	// -DGLIBCXX_DEBUG:
+	v.insert(v.begin(),
+		 db.positionlist_begin(1, ""),
+		 db.positionlist_end(1, ""));
+    }
+}
+
+// Test PostingIterator iterator_traits.
+DEFINE_TESTCASE(stlpostingiterator, !backend) {
+    Xapian::Database db;
+    vector<Xapian::docid> v;
+    // In Xapian <= 1.4.30 this gave a compile error with stdc++ and
+    // -DGLIBCXX_DEBUG:
+    v.insert(v.begin(), db.postlist_begin(""), db.postlist_end(""));
+}
+
+// Test TermIterator iterator_traits.
+DEFINE_TESTCASE(stltermiterator, !backend) {
+    Xapian::Document doc;
+    vector<string> v;
+    // In Xapian <= 1.4.30 this gave a compile error with stdc++ and
+    // -DGLIBCXX_DEBUG:
+    v.insert(v.begin(), doc.termlist_begin(), doc.termlist_end());
+}
+
+// Test Utf8Iterator iterator_traits.
+DEFINE_TESTCASE(stlutf8iterator, !backend) {
+    vector<unsigned> v;
+    // In Xapian <= 1.4.30 this gave a compile error with stdc++ and
+    // -DGLIBCXX_DEBUG:
+    v.insert(v.begin(), Xapian::Utf8Iterator(""), Xapian::Utf8Iterator());
+}
+
+// Test ValueIterator iterator_traits.
+DEFINE_TESTCASE(stlvalueiterator, !backend) {
+    Xapian::Document doc;
+    vector<string> v;
+    // In Xapian <= 1.4.30 this gave a compile error with stdc++ and
+    // -DGLIBCXX_DEBUG:
+    v.insert(v.begin(), doc.values_begin(), doc.values_end());
+}
