@@ -1,7 +1,7 @@
 /** @file
  * @brief Test the spelling correction suggestion API.
  */
-/* Copyright (C) 2007-2023 Olly Betts
+/* Copyright (C) 2007-2025 Olly Betts
  * Copyright (C) 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -316,3 +316,16 @@ DEFINE_TESTCASE(spell8, spelling) {
     // kin and kin used to cancel out in "skinking".
     TEST_EQUAL(db.get_spelling_suggestion("scimkin", 3), "skinking");
 }
+
+/// spell9 only present on git master.
+
+/// Regression test for bug due to rejecting based on matching n-gram count.
+DEFINE_TESTCASE(spell10, spelling) {
+    Xapian::Database db = get_database("spell10",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&) {
+					   wdb.add_spelling("Schiedsrichterstuhl", 1);
+					   wdb.add_spelling("Stuhl", 1);
+				       });
+    TEST_EQUAL(db.get_spelling_suggestion("Schtuhl", 3), "Stuhl");
+}
